# Copyright 2017 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""TFDecorator-aware replacements for the inspect module."""
import collections
import functools
import inspect as _inspect

import tensorflow.compat.v2 as tf

if hasattr(_inspect, "ArgSpec"):
    ArgSpec = _inspect.ArgSpec
else:
    ArgSpec = collections.namedtuple(
        "ArgSpec",
        [
            "args",
            "varargs",
            "keywords",
            "defaults",
        ],
    )

if hasattr(_inspect, "FullArgSpec"):
    FullArgSpec = _inspect.FullArgSpec
else:
    FullArgSpec = collections.namedtuple(
        "FullArgSpec",
        [
            "args",
            "varargs",
            "varkw",
            "defaults",
            "kwonlyargs",
            "kwonlydefaults",
            "annotations",
        ],
    )


def _convert_maybe_argspec_to_fullargspec(argspec):
    if isinstance(argspec, FullArgSpec):
        return argspec
    return FullArgSpec(
        args=argspec.args,
        varargs=argspec.varargs,
        varkw=argspec.keywords,
        defaults=argspec.defaults,
        kwonlyargs=[],
        kwonlydefaults=None,
        annotations={},
    )


if hasattr(_inspect, "getfullargspec"):
    _getfullargspec = _inspect.getfullargspec

    def _getargspec(target):
        """A python3 version of getargspec.

        Calls `getfullargspec` and assigns args, varargs,
        varkw, and defaults to a python 2/3 compatible `ArgSpec`.

        The parameter name 'varkw' is changed to 'keywords' to fit the
        `ArgSpec` struct.

        Args:
          target: the target object to inspect.

        Returns:
          An ArgSpec with args, varargs, keywords, and defaults parameters
          from FullArgSpec.
        """
        fullargspecs = getfullargspec(target)
        argspecs = ArgSpec(
            args=fullargspecs.args,
            varargs=fullargspecs.varargs,
            keywords=fullargspecs.varkw,
            defaults=fullargspecs.defaults,
        )
        return argspecs

else:
    _getargspec = _inspect.getargspec

    def _getfullargspec(target):
        """A python2 version of getfullargspec.

        Args:
          target: the target object to inspect.

        Returns:
          A FullArgSpec with empty kwonlyargs, kwonlydefaults and annotations.
        """
        return _convert_maybe_argspec_to_fullargspec(getargspec(target))


def currentframe():
    """TFDecorator-aware replacement for inspect.currentframe."""
    return _inspect.stack()[1][0]


def getargspec(obj):
    """TFDecorator-aware replacement for `inspect.getargspec`.

    Note: `getfullargspec` is recommended as the python 2/3 compatible
    replacement for this function.

    Args:
      obj: A function, partial function, or callable object, possibly decorated.

    Returns:
      The `ArgSpec` that describes the signature of the outermost decorator that
      changes the callable's signature, or the `ArgSpec` that describes
      the object if not decorated.

    Raises:
      ValueError: When callable's signature can not be expressed with
        ArgSpec.
      TypeError: For objects of unsupported types.
    """
    if isinstance(obj, functools.partial):
        return _get_argspec_for_partial(obj)

    decorators, target = tf.__internal__.decorator.unwrap(obj)

    spec = next(
        (
            d.decorator_argspec
            for d in decorators
            if d.decorator_argspec is not None
        ),
        None,
    )
    if spec:
        return spec

    try:
        # Python3 will handle most callables here (not partial).
        return _getargspec(target)
    except TypeError:
        pass

    if isinstance(target, type):
        try:
            return _getargspec(target.__init__)
        except TypeError:
            pass

        try:
            return _getargspec(target.__new__)
        except TypeError:
            pass

    # The `type(target)` ensures that if a class is received we don't return
    # the signature of its __call__ method.
    return _getargspec(type(target).__call__)


def _get_argspec_for_partial(obj):
    """Implements `getargspec` for `functools.partial` objects.

    Args:
      obj: The `functools.partial` object
    Returns:
      An `inspect.ArgSpec`
    Raises:
      ValueError: When callable's signature can not be expressed with
        ArgSpec.
    """
    # When callable is a functools.partial object, we construct its ArgSpec with
    # following strategy:
    # - If callable partial contains default value for positional arguments (ie.
    # object.args), then final ArgSpec doesn't contain those positional
    # arguments.
    # - If callable partial contains default value for keyword arguments (ie.
    # object.keywords), then we merge them with wrapped target. Default values
    # from callable partial takes precedence over those from wrapped target.
    #
    # However, there is a case where it is impossible to construct a valid
    # ArgSpec. Python requires arguments that have no default values must be
    # defined before those with default values. ArgSpec structure is only valid
    # when this presumption holds true because default values are expressed as a
    # tuple of values without keywords and they are always assumed to belong to
    # last K arguments where K is number of default values present.
    #
    # Since functools.partial can give default value to any argument, this
    # presumption may no longer hold in some cases. For example:
    #
    # def func(m, n):
    #   return 2 * m + n
    # partialed = functools.partial(func, m=1)
    #
    # This example will result in m having a default value but n doesn't. This
    # is usually not allowed in Python and can not be expressed in ArgSpec
    # correctly.
    #
    # Thus, we must detect cases like this by finding first argument with
    # default value and ensures all following arguments also have default
    # values. When this is not true, a ValueError is raised.

    n_prune_args = len(obj.args)
    partial_keywords = obj.keywords or {}

    args, varargs, keywords, defaults = getargspec(obj.func)

    # Pruning first n_prune_args arguments.
    args = args[n_prune_args:]

    # Partial function may give default value to any argument, therefore length
    # of default value list must be len(args) to allow each argument to
    # potentially be given a default value.
    no_default = object()
    all_defaults = [no_default] * len(args)

    if defaults:
        all_defaults[-len(defaults) :] = defaults

    # Fill in default values provided by partial function in all_defaults.
    for kw, default in partial_keywords.items():
        if kw in args:
            idx = args.index(kw)
            all_defaults[idx] = default
        elif not keywords:
            raise ValueError(
                "Function does not have **kwargs parameter, but "
                "contains an unknown partial keyword."
            )

    # Find first argument with default value set.
    first_default = next(
        (idx for idx, x in enumerate(all_defaults) if x is not no_default), None
    )

    # If no default values are found, return ArgSpec with defaults=None.
    if first_default is None:
        return ArgSpec(args, varargs, keywords, None)

    # Checks if all arguments have default value set after first one.
    invalid_default_values = [
        args[i]
        for i, j in enumerate(all_defaults)
        if j is no_default and i > first_default
    ]

    if invalid_default_values:
        raise ValueError(
            f"Some arguments {invalid_default_values} do not have "
            "default value, but they are positioned after those with "
            "default values. This can not be expressed with ArgSpec."
        )

    return ArgSpec(args, varargs, keywords, tuple(all_defaults[first_default:]))


def getfullargspec(obj):
    """TFDecorator-aware replacement for `inspect.getfullargspec`.

    This wrapper emulates `inspect.getfullargspec` in[^)]* Python2.

    Args:
      obj: A callable, possibly decorated.

    Returns:
      The `FullArgSpec` that describes the signature of
      the outermost decorator that changes the callable's signature. If the
      callable is not decorated, `inspect.getfullargspec()` will be called
      directly on the callable.
    """
    decorators, target = tf.__internal__.decorator.unwrap(obj)

    for d in decorators:
        if d.decorator_argspec is not None:
            return _convert_maybe_argspec_to_fullargspec(d.decorator_argspec)
    return _getfullargspec(target)


def getcallargs(*func_and_positional, **named):
    """TFDecorator-aware replacement for inspect.getcallargs.

    Args:
      *func_and_positional: A callable, possibly decorated, followed by any
        positional arguments that would be passed to `func`.
      **named: The named argument dictionary that would be passed to `func`.

    Returns:
      A dictionary mapping `func`'s named arguments to the values they would
      receive if `func(*positional, **named)` were called.

    `getcallargs` will use the argspec from the outermost decorator that
    provides it. If no attached decorators modify argspec, the final unwrapped
    target's argspec will be used.
    """
    func = func_and_positional[0]
    positional = func_and_positional[1:]
    argspec = getfullargspec(func)
    call_args = named.copy()
    this = getattr(func, "im_self", None) or getattr(func, "__self__", None)
    if ismethod(func) and this:
        positional = (this,) + positional
    remaining_positionals = [
        arg for arg in argspec.args if arg not in call_args
    ]
    call_args.update(dict(zip(remaining_positionals, positional)))
    default_count = 0 if not argspec.defaults else len(argspec.defaults)
    if default_count:
        for arg, value in zip(argspec.args[-default_count:], argspec.defaults):
            if arg not in call_args:
                call_args[arg] = value
    if argspec.kwonlydefaults is not None:
        for k, v in argspec.kwonlydefaults.items():
            if k not in call_args:
                call_args[k] = v
    return call_args


def getframeinfo(*args, **kwargs):
    return _inspect.getframeinfo(*args, **kwargs)


def getdoc(obj):
    """TFDecorator-aware replacement for inspect.getdoc.

    Args:
      obj: An object, possibly decorated.

    Returns:
      The docstring associated with the object.

    The outermost-decorated object is intended to have the most complete
    documentation, so the decorated parameter is not unwrapped.
    """
    return _inspect.getdoc(obj)


def getfile(obj):
    """TFDecorator-aware replacement for inspect.getfile."""
    unwrapped_object = tf.__internal__.decorator.unwrap(obj)[1]

    # Work around for the case when object is a stack frame
    # and only .pyc files are used. In this case, getfile
    # might return incorrect path. So, we get the path from f_globals
    # instead.
    if (
        hasattr(unwrapped_object, "f_globals")
        and "__file__" in unwrapped_object.f_globals
    ):
        return unwrapped_object.f_globals["__file__"]
    return _inspect.getfile(unwrapped_object)


def getmembers(obj, predicate=None):
    """TFDecorator-aware replacement for inspect.getmembers."""
    return _inspect.getmembers(obj, predicate)


def getmodule(obj):
    """TFDecorator-aware replacement for inspect.getmodule."""
    return _inspect.getmodule(obj)


def getmro(cls):
    """TFDecorator-aware replacement for inspect.getmro."""
    return _inspect.getmro(cls)


def getsource(obj):
    """TFDecorator-aware replacement for inspect.getsource."""
    return _inspect.getsource(tf.__internal__.decorator.unwrap(obj)[1])


def getsourcefile(obj):
    """TFDecorator-aware replacement for inspect.getsourcefile."""
    return _inspect.getsourcefile(tf.__internal__.decorator.unwrap(obj)[1])


def getsourcelines(obj):
    """TFDecorator-aware replacement for inspect.getsourcelines."""
    return _inspect.getsourcelines(tf.__internal__.decorator.unwrap(obj)[1])


def isbuiltin(obj):
    """TFDecorator-aware replacement for inspect.isbuiltin."""
    return _inspect.isbuiltin(tf.__internal__.decorator.unwrap(obj)[1])


def isclass(obj):
    """TFDecorator-aware replacement for inspect.isclass."""
    return _inspect.isclass(tf.__internal__.decorator.unwrap(obj)[1])


def isfunction(obj):
    """TFDecorator-aware replacement for inspect.isfunction."""
    return _inspect.isfunction(tf.__internal__.decorator.unwrap(obj)[1])


def isframe(obj):
    """TFDecorator-aware replacement for inspect.ismodule."""
    return _inspect.isframe(tf.__internal__.decorator.unwrap(obj)[1])


def isgenerator(obj):
    """TFDecorator-aware replacement for inspect.isgenerator."""
    return _inspect.isgenerator(tf.__internal__.decorator.unwrap(obj)[1])


def isgeneratorfunction(obj):
    """TFDecorator-aware replacement for inspect.isgeneratorfunction."""
    return _inspect.isgeneratorfunction(
        tf.__internal__.decorator.unwrap(obj)[1]
    )


def ismethod(obj):
    """TFDecorator-aware replacement for inspect.ismethod."""
    return _inspect.ismethod(tf.__internal__.decorator.unwrap(obj)[1])


def ismodule(obj):
    """TFDecorator-aware replacement for inspect.ismodule."""
    return _inspect.ismodule(tf.__internal__.decorator.unwrap(obj)[1])


def isroutine(obj):
    """TFDecorator-aware replacement for inspect.isroutine."""
    return _inspect.isroutine(tf.__internal__.decorator.unwrap(obj)[1])


def stack(context=1):
    """TFDecorator-aware replacement for inspect.stack."""
    return _inspect.stack(context)[1:]




# Copyright 2015 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Contains the GaussianDropout layer."""


import numpy as np
import tensorflow.compat.v2 as tf

from keras.src import backend
from keras.src.engine import base_layer
from keras.src.utils import tf_utils

# isort: off
from tensorflow.python.util.tf_export import keras_export


@keras_export("keras.layers.GaussianDropout")
class GaussianDropout(base_layer.BaseRandomLayer):
    """Apply multiplicative 1-centered Gaussian noise.

    As it is a regularization layer, it is only active at training time.

    Args:
      rate: Float, drop probability (as with `Dropout`).
        The multiplicative noise will have
        standard deviation `sqrt(rate / (1 - rate))`.
      seed: Integer, optional random seed to enable deterministic behavior.

    Call arguments:
      inputs: Input tensor (of any rank).
      training: Python boolean indicating whether the layer should behave in
        training mode (adding dropout) or in inference mode (doing nothing).

    Input shape:
      Arbitrary. Use the keyword argument `input_shape`
      (tuple of integers, does not include the samples axis)
      when using this layer as the first layer in a model.

    Output shape:
      Same shape as input.
    """

    def __init__(self, rate, seed=None, **kwargs):
        super().__init__(seed=seed, **kwargs)
        self.supports_masking = True
        self.rate = rate
        self.seed = seed

    def call(self, inputs, training=None):
        if 0 < self.rate < 1:

            def noised():
                stddev = np.sqrt(self.rate / (1.0 - self.rate))
                return inputs * self._random_generator.random_normal(
                    shape=tf.shape(inputs),
                    mean=1.0,
                    stddev=stddev,
                    dtype=inputs.dtype,
                )

            return backend.in_train_phase(noised, inputs, training=training)
        return inputs

    def get_config(self):
        config = {"rate": self.rate, "seed": self.seed}
        base_config = super().get_config()
        return dict(list(base_config.items()) + list(config.items()))

    @tf_utils.shape_type_conversion
    def compute_output_shape(self, input_shape):
        return input_shape




"""AUTOGENERATED. DO NOT EDIT."""

from keras.api._v2.keras.preprocessing import image
from keras.api._v2.keras.preprocessing import sequence
from keras.api._v2.keras.preprocessing import text
from keras.src.utils.image_dataset import image_dataset_from_directory
from keras.src.utils.text_dataset import text_dataset_from_directory
from keras.src.utils.timeseries_dataset import timeseries_dataset_from_array



# Copyright 2015 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================

"""Locally-connected layer for 2D input."""

from keras.src import activations
from keras.src import backend
from keras.src import constraints
from keras.src import initializers
from keras.src import regularizers
from keras.src.engine.base_layer import Layer
from keras.src.engine.input_spec import InputSpec
from keras.src.layers.locally_connected import locally_connected_utils
from keras.src.utils import conv_utils
from keras.src.utils import tf_utils

# isort: off
from tensorflow.python.util.tf_export import keras_export


@keras_export("keras.layers.LocallyConnected2D")
class LocallyConnected2D(Layer):
    """Locally-connected layer for 2D inputs.

    The `LocallyConnected2D` layer works similarly
    to the `Conv2D` layer, except that weights are unshared,
    that is, a different set of filters is applied at each
    different patch of the input.

    Note: layer attributes cannot be modified after the layer has been called
    once (except the `trainable` attribute).

    Examples:
    ```python
        # apply a 3x3 unshared weights convolution with 64 output filters on a
        32x32 image
        # with `data_format="channels_last"`:
        model = Sequential()
        model.add(LocallyConnected2D(64, (3, 3), input_shape=(32, 32, 3)))
        # now model.output_shape == (None, 30, 30, 64)
        # notice that this layer will consume (30*30)*(3*3*3*64) + (30*30)*64
        parameters

        # add a 3x3 unshared weights convolution on top, with 32 output filters:
        model.add(LocallyConnected2D(32, (3, 3)))
        # now model.output_shape == (None, 28, 28, 32)
    ```

    Args:
        filters: Integer, the dimensionality of the output space (i.e. the
          number of output filters in the convolution).
        kernel_size: An integer or tuple/list of 2 integers, specifying the
          width and height of the 2D convolution window. Can be a single integer
          to specify the same value for all spatial dimensions.
        strides: An integer or tuple/list of 2 integers, specifying the strides
          of the convolution along the width and height. Can be a single integer
          to specify the same value for all spatial dimensions.
        padding: Currently only support `"valid"` (case-insensitive). `"same"`
          will be supported in future. `"valid"` means no padding.
        data_format: A string, one of `channels_last` (default) or
          `channels_first`. The ordering of the dimensions in the inputs.
          `channels_last` corresponds to inputs with shape `(batch, height,
            width, channels)` while `channels_first` corresponds to inputs with
            shape
          `(batch, channels, height, width)`. When unspecified, uses
          `image_data_format` value found in your Keras config file at
          `~/.keras/keras.json` (if exists) else 'channels_last'.
          Defaults to 'channels_last'.
        activation: Activation function to use. If you don't specify anything,
          no activation is applied (ie. "linear" activation: `a(x) = x`).
        use_bias: Boolean, whether the layer uses a bias vector.
        kernel_initializer: Initializer for the `kernel` weights matrix.
        bias_initializer: Initializer for the bias vector.
        kernel_regularizer: Regularizer function applied to the `kernel` weights
          matrix.
        bias_regularizer: Regularizer function applied to the bias vector.
        activity_regularizer: Regularizer function applied to the output of the
          layer (its "activation").
        kernel_constraint: Constraint function applied to the kernel matrix.
        bias_constraint: Constraint function applied to the bias vector.
        implementation: implementation mode, either `1`, `2`, or `3`. `1` loops
          over input spatial locations to perform the forward pass. It is
          memory-efficient but performs a lot of (small) ops.  `2` stores layer
          weights in a dense but sparsely-populated 2D matrix and implements the
          forward pass as a single matrix-multiply. It uses a lot of RAM but
          performs few (large) ops.  `3` stores layer weights in a sparse tensor
          and implements the forward pass as a single sparse matrix-multiply.
            How to choose:
            `1`: large, dense models,
            `2`: small models,
            `3`: large, sparse models,  where "large" stands for large
              input/output activations (i.e. many `filters`, `input_filters`,
              large `np.prod(input_size)`, `np.prod(output_size)`), and "sparse"
              stands for few connections between inputs and outputs, i.e. small
              ratio `filters * input_filters * np.prod(kernel_size) /
              (np.prod(input_size) * np.prod(strides))`, where inputs to and
              outputs of the layer are assumed to have shapes `input_size +
              (input_filters,)`, `output_size + (filters,)` respectively. It is
              recommended to benchmark each in the setting of interest to pick
              the most efficient one (in terms of speed and memory usage).
              Correct choice of implementation can lead to dramatic speed
              improvements (e.g. 50X), potentially at the expense of RAM. Also,
              only `padding="valid"` is supported by `implementation=1`.
    Input shape:
        4D tensor with shape: `(samples, channels, rows, cols)` if
          data_format='channels_first'
        or 4D tensor with shape: `(samples, rows, cols, channels)` if
          data_format='channels_last'.
    Output shape:
        4D tensor with shape: `(samples, filters, new_rows, new_cols)` if
          data_format='channels_first'
        or 4D tensor with shape: `(samples, new_rows, new_cols, filters)` if
          data_format='channels_last'. `rows` and `cols` values might have
          changed due to padding.
    """

    def __init__(
        self,
        filters,
        kernel_size,
        strides=(1, 1),
        padding="valid",
        data_format=None,
        activation=None,
        use_bias=True,
        kernel_initializer="glorot_uniform",
        bias_initializer="zeros",
        kernel_regularizer=None,
        bias_regularizer=None,
        activity_regularizer=None,
        kernel_constraint=None,
        bias_constraint=None,
        implementation=1,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.filters = filters
        self.kernel_size = conv_utils.normalize_tuple(
            kernel_size, 2, "kernel_size"
        )
        self.strides = conv_utils.normalize_tuple(
            strides, 2, "strides", allow_zero=True
        )
        self.padding = conv_utils.normalize_padding(padding)
        if self.padding != "valid" and implementation == 1:
            raise ValueError(
                "Invalid border mode for LocallyConnected2D "
                '(only "valid" is supported if implementation is 1): ' + padding
            )
        self.data_format = conv_utils.normalize_data_format(data_format)
        self.activation = activations.get(activation)
        self.use_bias = use_bias
        self.kernel_initializer = initializers.get(kernel_initializer)
        self.bias_initializer = initializers.get(bias_initializer)
        self.kernel_regularizer = regularizers.get(kernel_regularizer)
        self.bias_regularizer = regularizers.get(bias_regularizer)
        self.activity_regularizer = regularizers.get(activity_regularizer)
        self.kernel_constraint = constraints.get(kernel_constraint)
        self.bias_constraint = constraints.get(bias_constraint)
        self.implementation = implementation
        self.input_spec = InputSpec(ndim=4)

    @property
    def _use_input_spec_as_call_signature(self):
        return False

    @tf_utils.shape_type_conversion
    def build(self, input_shape):
        if self.data_format == "channels_last":
            input_row, input_col = input_shape[1:-1]
            input_filter = input_shape[3]
        else:
            input_row, input_col = input_shape[2:]
            input_filter = input_shape[1]
        if input_row is None or input_col is None:
            raise ValueError(
                "The spatial dimensions of the inputs to "
                " a LocallyConnected2D layer "
                "should be fully-defined, but layer received "
                "the inputs shape " + str(input_shape)
            )
        output_row = conv_utils.conv_output_length(
            input_row, self.kernel_size[0], self.padding, self.strides[0]
        )
        output_col = conv_utils.conv_output_length(
            input_col, self.kernel_size[1], self.padding, self.strides[1]
        )
        self.output_row = output_row
        self.output_col = output_col

        if self.output_row <= 0 or self.output_col <= 0:
            raise ValueError(
                "One of the dimensions in the output is <= 0 "
                f"due to downsampling in {self.name}. Consider "
                "increasing the input size. "
                f"Received input shape {input_shape} which would produce "
                "output shape with a zero or negative value in a "
                "dimension."
            )

        if self.implementation == 1:
            self.kernel_shape = (
                output_row * output_col,
                self.kernel_size[0] * self.kernel_size[1] * input_filter,
                self.filters,
            )

            self.kernel = self.add_weight(
                shape=self.kernel_shape,
                initializer=self.kernel_initializer,
                name="kernel",
                regularizer=self.kernel_regularizer,
                constraint=self.kernel_constraint,
            )

        elif self.implementation == 2:
            if self.data_format == "channels_first":
                self.kernel_shape = (
                    input_filter,
                    input_row,
                    input_col,
                    self.filters,
                    self.output_row,
                    self.output_col,
                )
            else:
                self.kernel_shape = (
                    input_row,
                    input_col,
                    input_filter,
                    self.output_row,
                    self.output_col,
                    self.filters,
                )

            self.kernel = self.add_weight(
                shape=self.kernel_shape,
                initializer=self.kernel_initializer,
                name="kernel",
                regularizer=self.kernel_regularizer,
                constraint=self.kernel_constraint,
            )

            self.kernel_mask = (
                locally_connected_utils.get_locallyconnected_mask(
                    input_shape=(input_row, input_col),
                    kernel_shape=self.kernel_size,
                    strides=self.strides,
                    padding=self.padding,
                    data_format=self.data_format,
                )
            )

        elif self.implementation == 3:
            self.kernel_shape = (
                self.output_row * self.output_col * self.filters,
                input_row * input_col * input_filter,
            )

            self.kernel_idxs = sorted(
                conv_utils.conv_kernel_idxs(
                    input_shape=(input_row, input_col),
                    kernel_shape=self.kernel_size,
                    strides=self.strides,
                    padding=self.padding,
                    filters_in=input_filter,
                    filters_out=self.filters,
                    data_format=self.data_format,
                )
            )

            self.kernel = self.add_weight(
                shape=(len(self.kernel_idxs),),
                initializer=self.kernel_initializer,
                name="kernel",
                regularizer=self.kernel_regularizer,
                constraint=self.kernel_constraint,
            )

        else:
            raise ValueError(
                "Unrecognized implementation mode: %d." % self.implementation
            )

        if self.use_bias:
            self.bias = self.add_weight(
                shape=(output_row, output_col, self.filters),
                initializer=self.bias_initializer,
                name="bias",
                regularizer=self.bias_regularizer,
                constraint=self.bias_constraint,
            )
        else:
            self.bias = None
        if self.data_format == "channels_first":
            self.input_spec = InputSpec(ndim=4, axes={1: input_filter})
        else:
            self.input_spec = InputSpec(ndim=4, axes={-1: input_filter})
        self.built = True

    @tf_utils.shape_type_conversion
    def compute_output_shape(self, input_shape):
        if self.data_format == "channels_first":
            rows = input_shape[2]
            cols = input_shape[3]
        elif self.data_format == "channels_last":
            rows = input_shape[1]
            cols = input_shape[2]

        rows = conv_utils.conv_output_length(
            rows, self.kernel_size[0], self.padding, self.strides[0]
        )
        cols = conv_utils.conv_output_length(
            cols, self.kernel_size[1], self.padding, self.strides[1]
        )

        if self.data_format == "channels_first":
            return (input_shape[0], self.filters, rows, cols)
        elif self.data_format == "channels_last":
            return (input_shape[0], rows, cols, self.filters)

    def call(self, inputs):
        if self.implementation == 1:
            output = backend.local_conv(
                inputs,
                self.kernel,
                self.kernel_size,
                self.strides,
                (self.output_row, self.output_col),
                self.data_format,
            )

        elif self.implementation == 2:
            output = locally_connected_utils.local_conv_matmul(
                inputs,
                self.kernel,
                self.kernel_mask,
                self.compute_output_shape(inputs.shape),
            )

        elif self.implementation == 3:
            output = locally_connected_utils.local_conv_sparse_matmul(
                inputs,
                self.kernel,
                self.kernel_idxs,
                self.kernel_shape,
                self.compute_output_shape(inputs.shape),
            )

        else:
            raise ValueError(
                "Unrecognized implementation mode: %d." % self.implementation
            )

        if self.use_bias:
            output = backend.bias_add(
                output, self.bias, data_format=self.data_format
            )

        output = self.activation(output)
        return output

    def get_config(self):
        config = {
            "filters": self.filters,
            "kernel_size": self.kernel_size,
            "strides": self.strides,
            "padding": self.padding,
            "data_format": self.data_format,
            "activation": activations.serialize(self.activation),
            "use_bias": self.use_bias,
            "kernel_initializer": initializers.serialize(
                self.kernel_initializer
            ),
            "bias_initializer": initializers.serialize(self.bias_initializer),
            "kernel_regularizer": regularizers.serialize(
                self.kernel_regularizer
            ),
            "bias_regularizer": regularizers.serialize(self.bias_regularizer),
            "activity_regularizer": regularizers.serialize(
                self.activity_regularizer
            ),
            "kernel_constraint": constraints.serialize(self.kernel_constraint),
            "bias_constraint": constraints.serialize(self.bias_constraint),
            "implementation": self.implementation,
        }
        base_config = super().get_config()
        return dict(list(base_config.items()) + list(config.items()))




"""AUTOGENERATED. DO NOT EDIT."""

from keras.src.preprocessing.image import DirectoryIterator
from keras.src.preprocessing.image import ImageDataGenerator
from keras.src.preprocessing.image import Iterator
from keras.src.preprocessing.image import NumpyArrayIterator
from keras.src.preprocessing.image import apply_affine_transform
from keras.src.preprocessing.image import apply_brightness_shift
from keras.src.preprocessing.image import apply_channel_shift
from keras.src.preprocessing.image import random_brightness
from keras.src.preprocessing.image import random_channel_shift
from keras.src.preprocessing.image import random_rotation
from keras.src.preprocessing.image import random_shear
from keras.src.preprocessing.image import random_shift
from keras.src.preprocessing.image import random_zoom
from keras.src.utils.image_utils import array_to_img
from keras.src.utils.image_utils import img_to_array
from keras.src.utils.image_utils import load_img
from keras.src.utils.image_utils import save_img
from keras.src.utils.image_utils import smart_resize



# Copyright 2018 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Base class for recurrent layers backed by cuDNN."""


import tensorflow.compat.v2 as tf

from keras.src import backend
from keras.src.engine.input_spec import InputSpec
from keras.src.layers.rnn.base_rnn import RNN


class _CuDNNRNN(RNN):
    """Private base class for CuDNNGRU and CuDNNLSTM layers.

    Args:
      return_sequences: Boolean. Whether to return the last output
          in the output sequence, or the full sequence.
      return_state: Boolean. Whether to return the last state
          in addition to the output.
      go_backwards: Boolean (default False).
          If True, process the input sequence backwards and return the
          reversed sequence.
      stateful: Boolean (default False). If True, the last state
          for each sample at index i in a batch will be used as initial
          state for the sample of index i in the following batch.
      time_major: Boolean (default False). If true, the inputs and outputs will
          be in shape `(timesteps, batch, ...)`, whereas in the False case, it
          will be `(batch, timesteps, ...)`.
    """

    def __init__(
        self,
        return_sequences=False,
        return_state=False,
        go_backwards=False,
        stateful=False,
        time_major=False,
        **kwargs
    ):
        # We invoke the base layer's initializer directly here because we do not
        # want to create RNN cell instance.
        super(RNN, self).__init__(**kwargs)
        self.return_sequences = return_sequences
        self.return_state = return_state
        self.go_backwards = go_backwards
        self.stateful = stateful
        self.time_major = time_major
        self.supports_masking = False
        self.input_spec = [InputSpec(ndim=3)]
        if hasattr(self.cell.state_size, "__len__"):
            state_size = self.cell.state_size
        else:
            state_size = [self.cell.state_size]
        self.state_spec = [InputSpec(shape=(None, dim)) for dim in state_size]
        self.constants_spec = None
        self._states = None
        self._num_constants = 0
        self._vector_shape = tf.constant([-1])

    def call(self, inputs, mask=None, training=None, initial_state=None):
        if isinstance(mask, list):
            mask = mask[0]
        if mask is not None:
            raise ValueError("Masking is not supported for CuDNN RNNs.")

        # input shape: `(samples, time (padded with zeros), input_dim)`
        # note that the .build() method of subclasses MUST define
        # self.input_spec and self.state_spec with complete input shapes.
        if isinstance(inputs, list):
            initial_state = inputs[1:]
            inputs = inputs[0]
        elif initial_state is not None:
            pass
        elif self.stateful:
            initial_state = self.states
        else:
            initial_state = self.get_initial_state(inputs)

        if len(initial_state) != len(self.states):
            raise ValueError(
                "Layer has "
                + str(len(self.states))
                + " states but was passed "
                + str(len(initial_state))
                + " initial states."
            )

        if self.go_backwards:
            # Reverse time axis.
            inputs = backend.reverse(inputs, 1)
        output, states = self._process_batch(inputs, initial_state)

        if self.stateful:
            updates = [
                tf.compat.v1.assign(self_state, state)
                for self_state, state in zip(self.states, states)
            ]
            self.add_update(updates)

        if self.return_state:
            return [output] + states
        else:
            return output

    def get_config(self):
        config = {
            "return_sequences": self.return_sequences,
            "return_state": self.return_state,
            "go_backwards": self.go_backwards,
            "stateful": self.stateful,
            "time_major": self.time_major,
        }
        base_config = super(RNN, self).get_config()
        return dict(list(base_config.items()) + list(config.items()))

    @classmethod
    def from_config(cls, config):
        return cls(**config)

    @property
    def trainable_weights(self):
        if self.trainable and self.built:
            return [self.kernel, self.recurrent_kernel, self.bias]
        return []

    @property
    def non_trainable_weights(self):
        if not self.trainable and self.built:
            return [self.kernel, self.recurrent_kernel, self.bias]
        return []

    @property
    def losses(self):
        return super(RNN, self).losses

    def get_losses_for(self, inputs=None):
        return super(RNN, self).get_losses_for(inputs=inputs)

